/* evilwm - Minimalist Window Manager for X
 * Copyright (C) 1999-2000 Ciaran Anscomb <ciarana@rd.bbc.co.uk>
 * see README for license and other details. */

#include "evilwm.h"
#include "keys.h"
#include "events.h"
#include "client.h"
#include "screen.h"
#include "misc.h"
#include "newclient.h"
#include "mark.h"

#include <stdlib.h>
#include <stdio.h>
#include <sstream>

#include <X11/Xatom.h>
#include <X11/cursorfont.h>

Client* cl;
Keys keys;

void handle_key_event(XKeyEvent *e) {
    cl = current;

    keys.run(e);
}

void handle_button_event(XEvent *ev) {
	XButtonEvent* e = &ev->xbutton;

	cl = current;

	keys.run(e);

	XSync(dpy, False);
	XAllowEvents(dpy, ReplayPointer, CurrentTime);
	XSync(dpy, False);

/*
    if (c && e->window != root) {
		switch (e->button) {
		case Button1:
			move(c, 0); break;
		case Button2:
			resize(c, 0); break;
		case Button3:
			XLowerWindow(dpy, c->parent); break;
		}
    }
*/
}

void handle_configure_request(XConfigureRequestEvent *e) {
    Client *c = find_client(e->window);
    XWindowChanges wc;

    wc.sibling = e->above;
    wc.stack_mode = e->detail;
    if (c) {
		ungravitate(c);
		if (e->value_mask & CWX) c->x = e->x;
		if (e->value_mask & CWY) c->y = e->y;
		if (e->value_mask & CWWidth) c->width = e->width;
		if (e->value_mask & CWHeight) c->height = e->height;
		gravitate(c);

		wc.x = c->x - c->border;
		wc.y = c->y - c->border;
		wc.width = c->width + (c->border*2);
		wc.height = c->height + (c->border*2);
		wc.border_width = 0;
		XConfigureWindow(dpy, c->parent, e->value_mask, &wc);
		send_config(c);
    }

    wc.x = c ? c->border : e->x;
    wc.y = c ? c->border : e->y;
    wc.width = e->width;
    wc.height = e->height;
    XConfigureWindow(dpy, e->window, e->value_mask, &wc);
}

void handle_map_request(XMapRequestEvent *e) {
    Client *c = find_client(e->window);

    if (c) {
		unhide(c, RAISE);
    } else {
#ifdef DEBUG
		fprintf(stderr, "event:map->make_new_client(); ");
#endif
		make_new_client(e->window);
    }
}

void handle_unmap_event(XUnmapEvent* e) {
    Client* c = find_client(e->window);

    if (c) hide(c);
}

void handle_destroy_event(XDestroyWindowEvent *e) {
    Client *c = find_client(e->window);

    if (c) {
		//if (c == current) next(c);
		remove_client(c, NOT_QUITTING);
    }
}

void handle_client_message(XClientMessageEvent *e) {
#ifndef VWM
	Client *c = find_client(e->window);

	if (c && e->message_type == xa_wm_change_state &&
		e->format == 32 && e->data.l[0] == IconicState)
	{
		hide(c);
	}
#endif
}

void handle_property_change(XPropertyEvent *e) {
    Client *c = find_client(e->window);
    long dummy;

    if (c) {
		if (e->atom == XA_WM_NORMAL_HINTS)
			XGetWMNormalHints(dpy, c->window, c->size, &dummy);
    }
}

void handle_enter_event(XCrossingEvent *e) {
    Client *c = find_client(e->window);
    if (c) {
		if (c->vdesk != vdesk && c->vdesk != -1)
			return;
		XSetInputFocus(dpy, c->window, RevertToPointerRoot, CurrentTime);
		if (current && c!=current) {
			XSetWindowBackground(dpy, current->parent, bg.pixel);
			XClearWindow(dpy, current->parent);
		}
		XSetWindowBackground(dpy, c->parent, (c->vdesk==-1)?fc.pixel:fg.pixel);
		XClearWindow(dpy, c->parent);
		current = c;
    }
}

int string2int(const std::string& str) {
    std::istringstream iss(str);
    int ret;
    iss >> ret;
    return ret;
}

void do_move() {
    move(cl, 1);
    setmouse(cl->window, cl->width + cl->border - 1,
			 cl->height + cl->border - 1);
}

void move_window_x(const std::string& str) {
    if (cl != 0) {
		cl->x += string2int(str);
		do_move();
    }
}

void move_window_y(const std::string& str) {
    if (cl != 0) {
		cl->y += string2int(str);
		do_move();
    }
}

void do_resize() {
    resize(cl, 1);
    setmouse(cl->window, cl->width + cl->border - 1,
			 cl->height + cl->border - 1);
}

void resize_window_x(const std::string& str) {
    if (cl != 0) {
		cl->width += string2int(str);
		do_resize();
    }
}

void resize_window_y(const std::string& str) {
    if (cl != 0) {
		cl->height += string2int(str);
		do_resize();
    }
}

void move_window_upleft(const std::string&) {
    if (cl != 0) {
		cl->x = cl->border;
		cl->y = cl->border;
		do_move();
    }
}

void move_window_upright(const std::string&) {
    if (cl != 0) {
		cl->x = DisplayWidth(dpy, screen) - cl->width - cl->border;
		cl->y = cl->border;
		do_move();
    }
}

void move_window_downleft(const std::string&) {
    if (cl != 0) {
		cl->x = cl->border;
		cl->y = DisplayHeight(dpy, screen)
			- cl->height - cl->border;
		do_move();
    }
}

void move_window_downright(const std::string&) {
    if (cl != 0) {
		cl->x = DisplayWidth(dpy, screen) - cl->width - cl->border;
		cl->y = DisplayHeight(dpy, screen)
			- cl->height - cl->border;
		do_move();
    }
}

void delete_window(const std::string&) {
    if (cl != 0) {
		send_wm_delete(cl);
    }
}

void info_window(const std::string&) {
    if (cl != 0) {
		show_info(cl);
    }
}

void maximise_window(const std::string&) {
    if (cl != 0) {
		maximise_horiz(cl);
		maximise_vert(cl);
		resize(cl, 1);
		setmouse(cl->window, cl->width + cl->border - 1,
				 cl->height + cl->border - 1);
    }
}

void maximise_window_vert(const std::string&) {
    if (cl != 0) {
		maximise_vert(cl);
		resize(cl, 1);
		setmouse(cl->window, cl->width + cl->border - 1,
				 cl->height + cl->border - 1);
    }
}

void fix_window(const std::string&) {
    if (cl != 0) {
		if (cl->vdesk == vdesk || cl->vdesk == -1) {
			XSetWindowBackground(dpy, cl->parent,
								 cl->vdesk == -1 ? fg.pixel : fc.pixel);
			XClearWindow(dpy, cl->parent);
			cl->vdesk = cl->vdesk == -1 ? vdesk : -1;
		}
    }
}

void exec_command(const std::string& str) {
    spawn(str);
}

void next_focus(const std::string&) {
    next(cl);
}

void switch_vdesk(const std::string& str) {
    switch_vdesk(string2int(str));
}

void next_vdesk(const std::string&) {
    // XK_? is not good idea for vdesk number, I think.
    if (vdesk < opt_vd) switch_vdesk(vdesk + 1);
    else switch_vdesk(1);
}

void prev_vdesk(const std::string&) {
    if (vdesk > 1) switch_vdesk(vdesk - 1);
    else switch_vdesk(opt_vd);
}

void fix_next_vdesk(const std::string& str) {
    fix_window(str);
    next_vdesk(str);
    fix_window(str);
}

void fix_prev_vdesk(const std::string& str) {
    fix_window(str);
    prev_vdesk(str);
    fix_window(str);
}

void up_focus(const std::string&) {
    arrow_next(cl, 0, -1);
}

void down_focus(const std::string&) {
    arrow_next(cl, 0, 1);
}

void right_focus(const std::string&) {
    arrow_next(cl, 1, 0);
}

void left_focus(const std::string&) {
    arrow_next(cl, -1, 0);
}

void upleft_focus(const std::string&) {
    arrow_next(cl, -1, -1);
}

void upright_focus(const std::string&) {
    arrow_next(cl, 1, -1);
}

void downleft_focus(const std::string&) {
    arrow_next(cl, -1, 1);
}

void downright_focus(const std::string&) {
    arrow_next(cl, 1, 1);
}

void wm_quit(const std::string&) {
    quit_nicely();
}

void wm_restart(const std::string&) {
    restart();
}

void warp_pointer(int x, int y) {
	XWarpPointer(dpy, None, None, 0, 0, 0, 0, x, y);
}

void warp_pointer_x(const std::string& str) {
	warp_pointer(string2int(str), 0);
}

void warp_pointer_y(const std::string& str) {
	warp_pointer(0, string2int(str));
}

void mark_client_str(const std::string&) {
	if (cl) mark_client(cl);
}

void goto_mark_str(const std::string&) {
	goto_mark(cl);
}

void set_mode(const std::string& str) {
	keys.setMode(string2int(str));
}

void raise_window(const std::string&) {
	if (cl) XRaiseWindow(dpy, cl->parent);
}

void lower_window(const std::string&) {
	if (cl) XLowerWindow(dpy, cl->parent);
}

/*
void iconize(const std::string&) {
	if (cl) iconize(cl);
}
*/

void mouse_move(const std::string&) {
	if (cl) move(cl, 0);
}

void mouse_resize(const std::string&) {
	if (cl) resize(cl, 0);
}


